/**
 *
 * @author omozegieaziegbe
 */
package com.jcg;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.xml.sax.InputSource;
import java.io.StringReader;
import java.io.StringWriter;

public class XmlInsertNode {

    public static void main(String[] args) {
        try {
            // Sample XML strings
            String existingXml = "<company><employees></employees></company>";
            String newEmployeeXml = "<employee><name>Mr Fish</name><role>Developer</role><salary>75000</salary></employee>";

            // Parse the existing XML document
            Document existingDocument = convertStringToXMLDocument(existingXml);
            
            // Parse the new employee XML string
            Document newEmployeeDocument = convertStringToXMLDocument(newEmployeeXml);
            
            // Insert the new employee node into the existing document
            insertNodeIntoExistingDocument(existingDocument, newEmployeeDocument);
            
            // Display the updated document structure
            String updatedXml = documentToString(existingDocument);
            System.out.println("Updated Document Structure:\n" + updatedXml);

        } catch (Exception e) {
        }
    }
    
    /**
     * Converts a string containing XML into a Document object.
     */
    private static Document convertStringToXMLDocument(String xmlString) throws Exception {
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        DocumentBuilder builder = factory.newDocumentBuilder();
        InputSource inputSource = new InputSource(new StringReader(xmlString));
        return builder.parse(inputSource);
    }

    /**
     * Inserts the root node of the new Document into an existing Document.
     */
    private static void insertNodeIntoExistingDocument(Document existingDocument, Document newDocument) {
        // Get the root element of the new document (employee node)
        Node newEmployeeNode = newDocument.getDocumentElement();
        
        // Import the new node into the existing document
        Node importedNode = existingDocument.importNode(newEmployeeNode, true);
        
        // Append the imported node to the appropriate location (inside <employees>)
        existingDocument.getElementsByTagName("employees").item(0).appendChild(importedNode);
    }

    /**
     * Converts the XML Document into a string for display.
     */
    private static String documentToString(Document document) throws Exception {
        TransformerFactory transformerFactory = TransformerFactory.newInstance();
        Transformer transformer = transformerFactory.newTransformer();
        transformer.setOutputProperty(OutputKeys.INDENT, "yes");
        transformer.setOutputProperty("{http://xml.apache.org/xslt}indent-amount", "2");
        DOMSource source = new DOMSource(document);
        StringWriter writer = new StringWriter();
        StreamResult result = new StreamResult(writer);
        transformer.transform(source, result);
        return writer.toString();
    }
}
